using System;
using System.Security;
using System.Security.Permissions;
using System.Security.Principal;

//code moet minstens gebruik kunnen maken van de Principal klasse
[assembly:SecurityPermission(SecurityAction.RequestMinimum, ControlPrincipal=true)]

namespace SecuritySamples 
{

	public class IdentityCheck 
	{

		public static void Main() 
		{
			//zorgt ervoor dat het principal object wordt gevuld
			AppDomain.CurrentDomain.SetPrincipalPolicy(PrincipalPolicy.WindowsPrincipal);
			//lees principal object 
			WindowsPrincipal user = (WindowsPrincipal)System.Threading.Thread.CurrentPrincipal;

			Console.WriteLine("User name: {0}", user.Identity.Name);
			Console.WriteLine("Authentication type: {0}", user.Identity.AuthenticationType);
			Console.WriteLine("Is in Administrators group: {0}", user.IsInRole(WindowsBuiltInRole.Administrator));
			Console.WriteLine("Is in Guests group: {0}", user.IsInRole(WindowsBuiltInRole.Guest));
			//declaratieve checks
            try
            {
                Console.WriteLine("\n{0}", DeclAdminCheck());
            }
            catch (Exception e)
            {
                if (e is SecurityException & ((SecurityException)e).PermissionType == typeof(PrincipalPermission))
                {
                    Console.WriteLine("\nDeclarative check for Administrators group failed!");
                }
                else
                {
                    Console.WriteLine("\nException occurred: {0}\n", e);
                }
            }

            try
            {
                Console.WriteLine("\n{0}", DeclGuestCheck());
            }
            catch (Exception e)
            {
                if (e is SecurityException & ((SecurityException)e).PermissionType == typeof(PrincipalPermission))
                {
                    Console.WriteLine("\nDeclarative check for Guests group failed!");
                }
                else
                {
                    Console.WriteLine("\nException occurred: {0}\n", e);
                }
            }

            Console.Write("\nPress Enter to exit...");
            Console.Read();

		}
		//User moet lid zijn van administrators, anders geeft methode-aanroep exception
		[PrincipalPermission(SecurityAction.Demand, Role=@"BUILTIN\Administrators")]
		private static string DeclAdminCheck() 
		{
			return "Declarative check for Administrators group passed!";
		}
		//User moet lid zijn van guests, anders geeft methode-aanroep exception

		[PrincipalPermission(SecurityAction.Demand, Role=@"BUILTIN\Guests")]
		private static string DeclGuestCheck() 
		{
			return "Declarative check for Guests group passed!";
		}
	}
}
